/*
 * test-parser.c
 *
 * Copyright 1998, 1999 Michael Elizabeth Chastain, <mailto:mec@shout.net>.
 * Licensed under the Gnu Public License, Version 2.
 *
 * Small main program to test the parser.
 */

#include <stdio.h>
#include <stdlib.h>

#include "mconfig.h"
#include "parser.tab.h"

argument_type argument;

void block_show( const block_type *, int indent );



const char * op_name( op_type op )
{
    switch ( op )
    {
    case op_word:      return "";
    case op_equal:     return "=";
    case op_not_equal: return "!=";
    case op_and:       return "-a";
    case op_or:        return "-o";
    case op_not:       return "!";
    }

    error_enum_bogus( );
    return "BOGUS";
}



const char * verb_name( verb_type verb )
{
    switch ( verb )
    {
    case verb_NULL:		break;
    case verb_IF:		return "<IF>";
    case verb_MENU:		return "<MENU>";
    case verb_mainmenu_name:	return "mainmenu_name";
    case verb_comment:		return "comment";
    case verb_text:		return "text";
    case verb_unset:		return "unset";
    case verb_ask_bool:		return "bool";
    case verb_ask_hex:		return "hex";
    case verb_ask_int:		return "int";
    case verb_ask_string:	return "string";
    case verb_ask_tristate:	return "tristate";
    case verb_def_bool:		return "define_bool";
    case verb_def_hex:		return "define_hex";
    case verb_def_int:		return "define_int";
    case verb_def_string:	return "define_string";
    case verb_def_tristate:	return "define_tristate";
    case verb_dep_bool:		return "dep_bool";
    case verb_dep_hex:		return "dep_hex";
    case verb_dep_int:		return "dep_int";
    case verb_dep_string:	return "dep_string";
    case verb_dep_tristate:	return "dep_tristate";
    case verb_nchoice:		return "nchoice";
    }

    error_enum_bogus( );
    return "BOGUS";
}




void indent_show( int indent )
{
    int ispace;
    for ( ispace = 0; ispace < indent; ispace++ )
	printf( "  " );
}



void lexeme_show( const lexeme_type lexeme )
{
    if ( lexeme.squote )
	printf( "'" );
    printf( "%.*s", lexeme.piece.len, lexeme.piece.ptr );
    if ( lexeme.squote )
	printf( "'" );
}



void atom_show( const atom_type * atom )
{
    if ( atom->dollar )
	printf( "$" );
    lexeme_show( atom->lexeme );
}



void word_show( const word_type * word )
{
    if ( word->dquote )
	printf( "\"" );

    if ( word->literal.len >= 0 )
    {
	printf( "%.*s", word->literal.len, word->literal.ptr );
    }
    else
    {
	const atom_type * atom;

	for ( atom = word->atom_first; atom != NULL; atom = atom->next )
	    atom_show( atom );
    }

    if ( word->dquote )
	printf( "\"" );
}



void word_list_show( const word_list_type * word_list )
{
    const word_type * word;

    for ( word = word_list->first; word != NULL; word = word->next )
    {
	word_show( word );
	if ( word->next != NULL )
	    printf( " " );
    }
}



void prompt_show( const prompt_type * prompt )
{
    printf( "'%.*s'", prompt->value.len, prompt->value.ptr );
}



void symbol_show( const symbol_type * symbol )
{
    printf( "%.*s", symbol->name.len, symbol->name.ptr );
}



void expr_show( const expr_type * expr )
{
    switch ( expr->op )
    {
    default:
	error_enum_bogus( );
	break;

    case op_word:
	word_show( expr->word_left );
	break;

    case op_equal:    
    case op_not_equal:
	word_show( expr->word_left );
	printf( " %s ", op_name( expr->op ) );
	word_show( expr->word_right );
	break;

    case op_and:
    case op_or:
	expr_show( expr->expr_left );
	printf( " %s ", op_name( expr->op ) );
	expr_show( expr->expr_right );
	break;

    case op_not:
	printf( "%s ", op_name( expr->op ) );
	expr_show( expr->expr_left );
	break;
    }
}



void statement_show( const statement_type * statement, int indent )
{
    switch ( statement->verb )
    {
    default:
	error_enum_bogus( );
	break;

    case verb_IF:
	indent_show( indent );
	printf( "if [ " );
	expr_show( statement->sc_condition );
	printf( " ]; then\n" );
	block_show( statement->sc_block_left, indent+1 );
	if ( statement->sc_block_right != NULL )
	{
	    indent_show( indent );
	    printf( "else\n" );
	    block_show( statement->sc_block_right, indent+1 );
	}
	indent_show( indent );
	printf( "fi\n" );
	break;

    case verb_MENU:
	if ( indent > 0 )
	    printf( "\n" );

	indent_show( indent );
	printf( "MENU " );
	prompt_show( statement->sc_title );
	printf( "\n" );

	if ( indent > 0 )
	{
	    indent_show( indent+1 );
	    printf( "mainmenu_option next_comment\n" );
	}

	block_show( statement->sc_block_left, indent+1 );

	if ( indent > 0 )
	{
	    indent_show( indent+1 );
	    printf( "endmenu\n" );
	}

	break;

    case verb_unset:
	{
	    const symbol_type * symbol;

	    indent_show( indent );
	    printf( "%s", verb_name( statement->verb ) );

	    for ( symbol = statement->sb_symbol; symbol != NULL; symbol = symbol->next_list )
	    {
		printf( " " );
		symbol_show( symbol );
	    }

	    printf( "\n" );
	}
	break;

    case verb_mainmenu_name:
    case verb_comment:
    case verb_text:
    case verb_ask_bool:
    case verb_ask_hex:
    case verb_ask_int:
    case verb_ask_string:
    case verb_ask_tristate:
    case verb_def_bool:
    case verb_def_hex:
    case verb_def_int:
    case verb_def_string:
    case verb_def_tristate:
    case verb_dep_bool:
    case verb_dep_hex:
    case verb_dep_int:
    case verb_dep_string:
    case verb_dep_tristate:
	indent_show( indent );
	printf( "%s", verb_name( statement->verb ) );
	if ( statement->sb_prompt != NULL )
	    { printf( " " ); prompt_show( statement->sb_prompt ); }
	if ( statement->sb_symbol != NULL )
	    { printf( " " ); symbol_show( statement->sb_symbol ); }
	if ( statement->sb_def_value != NULL )
	    { printf( " " ); word_show( statement->sb_def_value ); }
	if ( statement->sb_dep_list != NULL
	&&   statement->sb_dep_list->first != NULL )
	{
	    printf( " " );
	    word_list_show( statement->sb_dep_list );
	}
	printf( "\n" );
	break;

    case verb_nchoice:
	{
	    const prompt_type * prompt;
	    const symbol_type * symbol;

	    indent_show( indent );
	    printf( "%s", verb_name( statement->verb ) );
	    printf( " " ); prompt_show( statement->sn_prompt );
	    printf( " " ); symbol_show( statement->sn_default_symbol );

	    for ( prompt  = statement->sn_ps_list->prompt_first,
	          symbol  = statement->sn_ps_list->symbol_first;
		  prompt != NULL && symbol != NULL;
		  prompt  = prompt->next, symbol = symbol->next_list )
	    {
		printf( " " ); prompt_show( prompt );
		printf( " " ); symbol_show( symbol );
	    }

	    printf( "\n" );
	}
	break;
    }
}



void block_show( const block_type * block, int indent )
{
    const statement_type * statement;

    for ( statement = block->first; statement != NULL; statement = statement->next )

	statement_show( statement, indent );
}



int main( int argc, char ** argv )
{
    int iarg;

    argument.cmdname = argv [0];
    argument.ds = "";

    for ( iarg = argc - 1; iarg > 0; --iarg )
    {
	const char * error_string;

	if ( input_push_file( argv [iarg], &error_string ) < 0 )
	    error_exit( error_string );
    }

#if 1
    parser_magic_cookie = COOKIE_CONFIG_LANGUAGE;
    if ( yyparse( ) != 0 )
	exit( 1 );
    block_show( parser_block_top, 0 );
#else
    parser_magic_cookie = COOKIE_DEF_CONFIG;
    if ( yyparse( ) != 0 )
	exit( 1 );
#endif

    return 0;
}



static int si = 0;

void scope_open( )
{
    indent_show( si++ );
    printf( "{\n"  );
}

void scope_close( )
{
    indent_show( --si );
    printf( "}\n" );
}

void scope_define( const symbol_type * symbol )
{
    indent_show( si );
    printf( "int %.*s = 0;\n", symbol->name.len, symbol->name.ptr );
}

void scope_use( piece_type piece )
{
    indent_show( si );
    printf( "foo( %.*s );\n", piece.len, piece.ptr );
}
